@@ -11,6 +11,11 @@
 #include <pwd.h>
 #include <time.h>
 #include <string.h>
+
+#ifdef _AIX
+#include <sys/resource.h>
+#endif
+
 #include <sys/wait.h>
 
 #include <X11/Intrinsic.h>
@@ -547,7 +552,10 @@ static int devAction(int d, char *action)
     if (resources.echo_actions)
       fprintf(stderr, "%s\n", action);
     freopen("/dev/null", "r", stdin);
-    execlp(user.shell, user.shell, "-c", action, NULL);
+    if (user.arg0flag)
+      execlp(user.shell, user.shell, "-c", action, user.shell, NULL);
+    else
+      execlp(user.shell, user.shell, "-c", action, NULL);
     perror("Exec failed");
     exit(1);
   } else if (waitpid(pid, &status, 0) == -1 || !WIFEXITED(status) ||
@@ -678,7 +686,7 @@ static void createTextDisplay(FileWindowRec *fw)
       if((pw = getpwuid(fw->files[i]->stats.st_uid)) == NULL) {
 	char tmp[11];
  	
-	sprintf(tmp, "%d", fw->files[i]->stats.st_uid);
+	sprintf(tmp, "%lu", (unsigned long) fw->files[i]->stats.st_uid);
 	owners[i] = XtNewString(tmp);
       }
       else
@@ -706,8 +714,10 @@ static void createTextDisplay(FileWindowRec *fw)
       text_toggle_args[2].value = (XtArgVal) file->name;
       if (file->stats.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
 	text_toggle_args[5].value = (XtArgVal) exec_translations;
-      else
+      else {
+        file->type = fileType(file->name); /* needed for push/drop-actions */
 	text_toggle_args[5].value = (XtArgVal) file_translations;
+      }
     }
 
     file->icon.form = XtCreateManagedWidget(file->name,
