@@ -174,13 +174,8 @@ static int setupMoveCopy(FileWindowRec *fw, int i)
 
 static void fileEndMove(FileWindowRec *fw, int i)
 {
-  char *fromdir, *todir;
-
   if (!setupMoveCopy(fw, i)) return;
 
-  fromdir = strcpy(alloca(fromi+1), from);
-  todir = strcpy(alloca(toi+1), to);
-
   if (!strcmp(from, to)) {
     error("Move:", "Source and destination are identical");
     return;
@@ -204,7 +199,7 @@ static void fileEndMove(FileWindowRec *fw, int i)
   }
 
   if (moveFiles()) {
-    markForUpdate(fromdir); markForUpdate(todir);
+    markForUpdate(from); markForUpdate(to);
     intUpdate();
   }
 
@@ -216,12 +211,8 @@ static void fileEndMove(FileWindowRec *fw, int i)
 
 static void fileEndCopy(FileWindowRec *fw, int i)
 {
-  char *todir;
-
   if (!setupMoveCopy(fw, i)) return;
 
-  todir = strcpy(alloca(toi+1), to);
-
   if (!strcmp(from, to)) {
     error("Copy:", "Source and destination are identical");
     return;
@@ -245,7 +236,7 @@ static void fileEndCopy(FileWindowRec *fw, int i)
   }
 
   if (copyFiles()) {
-    markForUpdate(todir);
+    markForUpdate(to);
     intUpdate();
   }
 
@@ -262,7 +253,7 @@ static void fileEndExec(FileWindowRec *fw, int i)
   char **argv;
 
   l = strlen(fw->directory);
-  path = alloca(l+strlen(fw->files[i]->name)+2);
+  path = (char *)alloca(l+strlen(fw->files[i]->name)+2);
   strcpy(path, fw->directory);
   if (path[l-1] != '/')
     path[l++] = '/';
@@ -290,7 +281,7 @@ static void fileEndAction(FileWindowRec *fw, int i)
   if (!action) return;
 
   l = strlen(fw->directory);
-  path = alloca(l+strlen(fw->files[i]->name)+2);
+  path = (char *)alloca(l+strlen(fw->files[i]->name)+2);
   strcpy(path, fw->directory);
   if (path[l-1] != '/')
     path[l++] = '/';
@@ -552,9 +543,9 @@ void fileHighlight(Widget w, XEvent *event, String *params,
     return;
   }
 
-  if (i == -1 && !permission(&fw->stats,W_OK) || 
-      i != -1 && S_ISDIR(fw->files[i]->stats.st_mode) &&
-      !permission(&fw->files[i]->stats, P_WRITE))
+  if ((i == -1 && !permission(&fw->stats,W_OK)) || 
+      (i != -1 && S_ISDIR(fw->files[i]->stats.st_mode) &&
+      !permission(&fw->files[i]->stats, P_WRITE)))
     XChangeActivePointerGrab(XtDisplay(w),
 		       EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
 		       curs[NOENTRY_CUR], CurrentTime);
@@ -736,7 +727,8 @@ void fileBeginDrag(Widget w, XEvent *event, String *params,
 void fileExecFile(Widget w, XEvent *event, String *params, 
 		  Cardinal *num_params)
 {
-  int i;
+  int i, l;
+  char *path;
   char **argv;
   FileWindowRec *fw;
 
@@ -748,10 +740,16 @@ void fileExecFile(Widget w, XEvent *event, String *params,
     return;
   }
 
+  l = strlen(fw->directory);
+  path = (char *)alloca(l+strlen(fw->files[i]->name)+2);
+  strcpy(path, fw->directory);
+  if (path[l-1] != '/')
+    path[l++] = '/';
+  strcpy(path+l, fw->files[i]->name);
   argv = (char **) XtMalloc(2 * sizeof(char *));
   argv[0] = XtNewString(fw->files[i]->name);
   argv[1] = NULL;
-  executeApplication(fw->files[i]->name, fw->directory, argv);
+  executeApplication(path, fw->directory, argv);
 
   freeArgv(argv);
 }
@@ -792,17 +790,20 @@ void fileExecAction(Widget w, XEvent *event, String *params,
 	readApplicationData(resources.app_file);
 	updateApplicationDisplay();
       } else {
+        int k = 0;
 	char *action = varPopup(fw->files[i]->type->icon_bm,
 				fw->files[i]->type->push_action);
 
 	if (!action) return;
 
-	argv = (char **) XtMalloc(5 * sizeof(char *));
-	argv[0] = user.shell;
-	argv[1] = "-c";
-	argv[2] = action;
-	argv[3] = fw->files[i]->name;
-	argv[4] = NULL;
+	argv = (char **) XtMalloc( (user.arg0flag ? 6 : 5) * sizeof(char *));
+	argv[k++] = user.shell;
+	argv[k++] = "-c";
+	argv[k++] = action;
+        if (user.arg0flag)
+          argv[k++] = user.shell;
+	argv[k++] = fw->files[i]->name;
+	argv[k] = NULL;
 
 	executeApplication(user.shell, fw->directory, argv);
 
